// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.

import React from 'react';


const StepsIcon = (): JSX.Element => {
    return (
        <svg width="20" height="24" viewBox="0 0 4 14" fill="none" xmlns="http://www.w3.org/2000/svg">
            <rect x="0.25" y="0.25" width="3.5" height="3.5" rx="0.25" stroke="black" strokeWidth="0.5"/>
            <rect x="0.25" y="5.25" width="3.5" height="3.5" rx="0.25" stroke="black" strokeWidth="0.5"/>
            <rect x="0.25" y="10.25" width="3.5" height="3.5" rx="0.25" stroke="black" strokeWidth="0.5"/>
        </svg>
    )
}

export default StepsIcon;
