/* 
    Contains the css for the column header cards which display an 
    individual Column Header that is used as a key in the row or column 
    section of the pivot table
*/

.pivot-table-column-header-card-div {
    height: 30px;
    align-items: center;

    display: flex;
    flex-direction: row;
    justify-content: space-between;
    
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;

    background-color: #F8F8F8;

    margin-top: 5px;
}

.pivot-table-column-header-card-div-text {
    font-size: 14px;
    margin-left: 5px;
    overflow: hidden;
    text-overflow: ellipsis; /* add ... for cut off text */
}

.pivot-table-column-header-card-exit-div {
    margin-right: 15px;
}

