

.custom-dropdown {
    z-index: 3;
    position: absolute;

    /* 
        We manually position this popup currently, which we will need
        to change when we use this popup in places other than just
        the step list view. We will make these parameters to style
        on the component!
    */
    right: 0px;
    top: 27px;

    padding: 10px;

    background: white;
    border-radius: 1px;

    text-align: left;

    box-shadow: 0 0px 8px 0 rgba(0, 0, 0, 0.2), 0 6px 20px 0 rgba(0, 0, 0, 0.19);
}

.custom-dropdown > div:not(:first-child) {
    margin-top: 10px;
}

.custom-dropdown > div:hover {
    text-decoration: underline;
}

.custom-dropdown-inactive {
    color: grey;
}
