.default-taskpane-div {
    height: 100%;
    width: 100%;
    display: flex;
    flex-direction: column;

    /* add standard spacing around the entire taskpane */
    padding-left: 10px;
    padding-right: 10px;
    padding-top: 5px;
    padding-bottom: 5px;

    /* set standard taskpane color */ 
    color: #343434;
    /* We have to set the background, as the taskpane turns black on fullscreen for some reason */
    background-color: white;

    /* stylize border of box */ 
    border: 1px solid #9e9e9e;
    box-sizing: border-box;
}

.default-taskpane-header-div {
    width: 100%;
    height: 25px;
    display: flex;
    flex-direction: row;
    justify-content: space-between;
}

.default-taskpane-header-text {
    font-size: 20px;
    width: 100%;
}

.default-taskpane-body-div {
    display: flex;
    flex-direction: column;

    height: 100%;
    overflow-y: scroll;

    margin-top: 15px;
}

.default-taskpane-body-section-div {
    margin-top: 10px;
}

.default-taskpane-body-section-title-text {
    font-size: 16px;
}

