// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.

import React from 'react';


const DropdownIcon = (): JSX.Element => {
    return (
        <svg width="20" height="16" viewBox="0 0 4 16" fill="none" xmlns="http://www.w3.org/2000/svg">
            <circle cx="2" cy="2" r="2" fill="#343434"/>
            <circle cx="2" cy="8" r="2" fill="#343434"/>
            <circle cx="2" cy="14" r="2" fill="#343434"/>
        </svg>
    )
}

export default DropdownIcon;

