.column-describe-table-container {
    display: flex;
    flex-direction: column;
    border: 1px black solid;
    border-radius: 3px;

    margin: 10px;
    padding: 5px 20px;
}

.column-describe-table-row {
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    padding: 5px;
}

.column-describe-chart-row:not(:last-of-type) {
    border-bottom: 1px gray solid;
}