

.raw-python-textarea {
    width: 100%;
    /* Don't let users expand to big and get stuck */
    resize: vertical;
    max-height: 150px;
}

.import-files-select {
    display: flex;
    flex-direction: column;
}

/* Make the select children center themselves */
.import-files-select > select {
    margin: auto;
    margin-bottom: 10px;
}

.import-files-old-and-new-container {
    display: flex;
    flex-direction: row;
}

.previous-file-name-labels-container {
    display: flex;
    flex-direction: column;
    margin-right: 10px;
    width: 50%;
}

.previous-file-name-label {
    font-size: 14px;
    line-height: 20px;
    color: #343434;

    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis; /* add ... for cut off text */

    margin-bottom: 15px;
    margin-top: 5px;
}