/* css for MitoSheet */
.mitosheet-ag-grid {
    height: 100%;
    width: 100%;
}

/* 
    Overwrite the default line height so that the text is centered
    inside of the shorter cells
*/ 
.ag-theme-alpine .ag-cell {
    line-height: 23px; /* to center the text inside the cell with the 1 px margin around it */
}

/* 
    Overwrite the height of the cell editor set by .ag-theme-alpine .ag-cell-inline-editing
*/
.mito-cell-editor {
    height: 25px !important;
}

/* 
    Overwrite the default background color that is set when by allowing the user to select multiple rows
*/
.ag-row-selected {
    background-color:rgba(152, 205, 248, 0.3) !important
}
