# Copyright (c) Mito.
# Distributed under the terms of the Modified BSD License.

# AUTOGENERATED | DO NOT EDIT BELOW
version_info = (0, 1, 158)
# AUTOGENERATED | DO NOT EDIT ABOVE
__version__ = ".".join(map(str, version_info))