.control-panel-taskpane-container {
    overflow-y: hidden;
    height: 100%;

    border: 1px solid #9e9e9e;
    box-sizing: border-box;

    color: #343434;
    background: white;
}

.control-panel-taskpane-tab-container {
    /* 
        We put the tabs at the bottom of the taskpane container,
        which they are the child of. This way, they are always visible
    */
    position: absolute;
    bottom: 30px;

    display: flex;
    justify-content: space-around;

    width: inherit;
}

.control-panel-taskpane-tab {
    width: 100%;
    border: 1px #343434 solid;
    background: white;
    text-align: center;
    padding: 4px;
}

/* When selected, white. when not selected, this color. */
.control-panel-taskpane-tab.selected {
    background: #FFFFFF;
    border-bottom: 3px solid #0081DE;
}

/* When selected, white. when not selected, this color. */
.control-panel-taskpane-tab.unselected {
    background: #FFFFFF;
}

.sort-buttons-div {
    display: flex;
    flex-direction: row wrap;
    justify-content: space-evenly;
}

.sort-button {
    padding-top: 1vh;
    padding-bottom: 1vh;
    padding-left: 2vw;
    padding-right: 2vw;

    font-size: 12px;
    background-color: #F8F8F8;
    border-radius: 3px;

    border-style: solid; /* get rid of shadow around button*/
}

.sort-button-selected{
    color: #0081DE;
    border-color: #0081DE;
}

.sort-button-unselected {
    color: #929292;
    border-color: #DDDDDD;
}

.filter-modal-section-spacer {
    margin-top: 15px;
}

.filter-modal-section-title {
    display: flex;
    flex-direction: row;
    justify-content: space-between;

    font-size: 14px;
    font-weight: bold;
    color: #343434;
    text-align: left;
    margin-bottom: 5px;
}

.filter-modal-centering-container {
    text-align: center;
    align-items: center;
}