.small-select {
    background-color: #F8F8F8;
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;

    width: 60px;
    height: 30px;
}

/* set the style of the select dropdown*/ 
.small-select option {
    background-color:#F8F8F8;
    color: #343434;
}

/* change font size of options text*/
optgroup { 
    font-size:14px; 
}


