/* 
    Contains the css for the value aggregations cards which display the
    aggregation method and column header of each value of the pivot table
*/

.pivot-table-value-aggregation-card-aggregation-info-div {
    height: 30px;
    align-items: center;

    display: flex;
    flex-direction: row;
    justify-content: left;
    width: 80%;
}

.pivot-table-value-aggregation-card-div-text {
    font-size: 14px;
    margin-left: 5px;
    width: 80%;
    white-space: nowrap; /* don't let the text wrap */
    overflow: hidden;
    text-overflow: ellipsis; /* add ... for cut off text */
}