.filter-object {
    width: 100%;
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    flex-wrap: nowrap;
    text-align: center;

    margin-bottom: 10px;
}

/* Set the the left margin for every item except the first */
.filter-object .filter-object-item + .filter-object-item {
    margin-left: 15px;
}

.filter-where-label {
    width: 50px;
    min-width: 50px; /* Set the min-width because the text is less than 50px wide*/
    /* Should be the same width as filter-select */
    text-align: center; 
    color: #343434;
    flex-grow: 0;
}

.filter-select {
    color: #343434;

    background: #F8F8F8;
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;
    
    width: 50px;
    height: 25px; 
}

/* Set the style of the select dropdown */ 
.filter-select option {
    background-color:#F8F8F8;
    color: #343434;
}

.filter-operator {
    flex-grow: 0;
}

.filter-condition {
    flex-grow: 2;
}

.filter-value {
    flex-grow: 3;
}

/* Make the filter close button div a flex box to vertically center*/
.filter-close-button {
    display: flex;
    flex-direction: column;
    justify-content: center
}

.filter-card-buttons-container {
    width: 100%;
    text-align: left;
}

.filter-group {
    width: 100%;

    background: #F8F8F8;
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;

    padding: 10px 5px;

    /* Add a margin to the between the filter group and the next item*/
    margin-bottom: 10px; 
}

.filter-modal-centering-container > .filter-object > .filter-close-button {
    /* 
        Only add margin-right to non-FilterGroups to get the X's to line up.
        The margin-right should be the same as the filter-group padding-right
    */
    margin-right: 5px;
}

.filter-group-and-operator-div {
    width: 100%;
    display: flex;
    flex-direction: row;
    flex-wrap: nowrap;
}

.filter-add-filter-button {
    /* Center the text inside the button */
    display: flex;
    flex-direction: column;
    justify-content: space-around;

    /* Stylize the button */
    background: #F8F8F8;
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;

    /* Stylize the text */
    color: #0081DE;
    font-size: 14px;
    text-align: left;
    padding-left: 5px;

    /* Set dimensions */
    width: 110px;
    height: 20px;
}

.filter-add-filter-button > option {
    color: #343434;
}