
/* Contains CSS for the MultiToggleBox element */

.multi-toggle-box {
    /* These in pair make it so less than the max height will just take up the space it needs, and anything larger scrolls */
    max-height: 80px;
    overflow: auto;

    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;
    background-color: #F8F8F8;
}