

.sheet-tab-bar {
    /* Deal with the overflow */
    overflow-x: auto;
    overflow-y: clip;
    white-space: nowrap;
}

.sheet-shape {
    padding: 5px;
    color: grey;
    border-left: 1px #C2C2C2 solid;

    /* A bit of a hack to make the text centered */
    line-height: 20px;
}

.tab {
    /* set background color and border of sheet tab*/
    background: #FDFDFD;
    border: 1px solid #C2C2C2;
    box-sizing: border-box;

    height: 100%;
    padding: 5px 10px 5px 10px;

    display: inline-block;
    text-align: center;
    text-decoration: none;
}

.selected-tab {
    border-bottom: 3px solid #0081DE;
    box-sizing: border-box;
}

.tab-text {
    display: flex;
    justify-content: center;
}

.tab-sheet-name {
    margin-right: 5px;
}

/* add a mito blue bar to the bottom of the selected sheet tab */
.selected-tab-indicator {
    border: 2px solid #0081DE;
    background-color: #0081DE;
}

.sheet-tab-actions-dropdown {
    z-index: 2;
    position: absolute;
    top: calc(100% - 125px);

    background-color: white;
    box-shadow: 0px 0px 4px #888;
    border-radius: 3px;

    text-align: left;

    display: flex;
    flex-direction: column;
}

.sheet-tab-action {
    padding: 5px 10px 5px 10px;
}

.sheet-tab-action:first-child {
    margin-top: 5px;
}

.sheet-tab-action:last-child {
    margin-bottom: 5px;
}

.sheet-tab-action:hover {
    background-color: #C2C2C2;
}