.column-name-card-container {
    display: inline-block;
    margin-bottom: 10px;
    margin-top: 2px;
    width: 100%;
}

.column-name-card-column-name-div {
    display: inline-flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: baseline;
    width: 85%;
}

.column-name-card-input-text {
    font-size: 14px;
    line-height: 14px;
    width: 85%
}

.column-name-card-column-header {
    font-size: 20px;
    width: 100%;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis; /* add ... for cut off text */
}

.column-name-card-column-header:hover {
    text-decoration: underline;
}
