.select {
    background-color: #F8F8F8;
    border: 0.5px solid #343434;
    box-sizing: border-box;
    border-radius: 3px;
    height: 30px;
}

.large-select {
    width: 130px;
}

.extra-large-select {
    width: 100%;
}

/* set the style of the select dropdown itself */ 
.large-select option {
    background-color:#F8F8F8;
}

/* change font size of options text*/
optgroup { 
    font-size:14px; 
}