

.step-taskpane-step-container {
    display: flex;
    justify-content: space-between;

    margin-right: 10px;

    /* 
        We position the container so that the absolute positioning of the dropdown
        correctly locks to this!
    */
    position: relative;
    width: 100%;
}

.step-taskpane-step-container-left {
    width: 80%;
    display: flex;
}

.step-taskpane-step-container-right {
    margin-top: 10px;
}

.step-taskpane-step-icon {
    min-width: 30px;
    margin-top: 6px;
}

.step-taskpane-missing-icon {
    font-weight: bold;
}

.step-taskpane-step-text {
    width: 100%;
}

.step-taskpane-step-header {
    font-weight: bold;
    font-size: 15px;
}

.step-taskpane-step-subtext {
    /* The subtext can get long, so we allow users to scroll the subtext in this case */
    overflow-x: scroll;
    white-space: nowrap;
    padding-bottom: 10px;
    
    color: grey;
}